package de.intevation.gnv.math;

import org.apache.commons.math.optimization.fitting.ParametricRealFunction;

import org.apache.commons.math.FunctionEvaluationException;

import org.apache.commons.math.analysis.UnivariateRealFunction;

/**
 *  @author Sascha L. Teichmann
 */
public class LinearFunction
implements   ParametricRealFunction
{
    public static final LinearFunction INSTANCE = new LinearFunction();

    public static class Univariate 
    implements          UnivariateRealFunction
    {
        protected double m;
        protected double b;

        public Univariate() {
        }

        public Univariate(double x1, double y1, double x2, double y2) {
            if (y1 == y2) {
                m = 0d;
                b = (x1 + x2)*0.5d;
            }
            else {
                m = (x1 - x2)/(y1 - y2);
                b = y1 - m*x1;
            }
        }

        public double value(double x) {
            return m*x + b;
        }
    } // class Univariate

    public LinearFunction() {
    }

    public double value(double x, double [] parameters)
    throws FunctionEvaluationException 
    {
        return x*parameters[0] + parameters[1];
    }

    public double [] gradient(double x, double [] parameters) 
    throws FunctionEvaluationException 
    {
        return new double [] { x, 1f };
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
